/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// scalarDirichletBC.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000


/*****************/
/*****************/

#include "Box.H"
#include "FArrayBox.H"
#include "REAL.H"
#include "SPACE.H"
#include "Tuple.H"
#include "LoHiSide.H"
#include "scalarDirichletBC.H"
#include "MayDay.H"



void
scalarDirichletBC::fillBCValues(FArrayBox& a_neumfac,
                          FArrayBox& a_dircfac,
                          FArrayBox& a_inhmval,
                          Real a_dx,
                          const Box& a_domain) const
{
  ProblemDomain physdomain(a_domain);
  fillBCValues(a_neumfac, a_dircfac, a_inhmval, a_dx, physdomain);
}


void
scalarDirichletBC::fillBCValues(FArrayBox& a_neumfac,
                          FArrayBox& a_dircfac,
                          FArrayBox& a_inhmval,
                          Real a_dx,
                          const ProblemDomain& a_domain) const
{
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(0.0);
  a_dircfac.setVal(1.0);
  a_inhmval.setVal(m_BCVal);
}

BoxGhostBC* 
scalarDirichletBC::new_boxghostbc() const
{
  scalarDirichletBC* newop = new scalarDirichletBC();
  if(newop == NULL)
    {
      MayDay::Error("Out of Memory in scalarDirichletBC::new_boxghostbc");
    }

  newop->setVal(m_BCVal);
  return static_cast<BoxGhostBC*>(newop);
}


scalarDirichletBC::scalarDirichletBC() : BoxGhostBC()
{
  m_BCVal = 1.0e8;
}

scalarDirichletBC::~scalarDirichletBC()
{
}

scalarDirichletBC::scalarDirichletBC(int dir, Side::LoHiSide side)
    : BoxGhostBC(dir,side)
{
  m_BCVal = 1.0e8;
}

scalarDirichletBC::scalarDirichletBC(int dir, Side::LoHiSide side, const Interval& a_comps)
    : BoxGhostBC(dir,side,a_comps)
{
  m_BCVal = 1.0e8;

}


void
scalarDirichletBC::setVal(const Real a_bcVal) 
{
  m_BCVal = a_bcVal;
}


scalarInflowBC::scalarInflowBC() : BoxGhostBC()
{
}

scalarInflowBC::scalarInflowBC(int dir, 
			       Side::LoHiSide sd) 
  : BoxGhostBC(dir,sd)
{

}

scalarInflowBC::scalarInflowBC(int dir, 
			       Side::LoHiSide sd,
			       const Interval& a_comps) 
  : BoxGhostBC(dir,sd,a_comps)
{
}

scalarInflowBC::~scalarInflowBC()
{
}

void
scalarInflowBC::setScalarType(int a_scalType)
{
  m_scalType = a_scalType;
}

int
scalarInflowBC::scalarType() const
{
  return m_scalType;
}

void 
scalarInflowBC::isHomogeneous(bool a_isHomogeneous)
{
  m_isHomogeneous = a_isHomogeneous;
}

bool
scalarInflowBC::isHomogeneous() const
{
  return m_isHomogeneous;
}


void
scalarInflowBC::fillBCValues(FArrayBox& a_neumfac,
			     FArrayBox& a_dircfac,
			     FArrayBox& a_inhmval,
			     Real a_dx,
			     const Box& a_domain) const
{
  ProblemDomain physdomain(a_domain);
  fillBCValues(a_neumfac, a_dircfac, a_inhmval, a_dx, physdomain);
}

void
scalarInflowBC::fillBCValues(FArrayBox& a_neumfac,
			     FArrayBox& a_dircfac,
			     FArrayBox& a_inhmval,
			     Real a_dx,
			     const ProblemDomain& a_domain) const
  {
  // this will need to be reworked a little.
  assert(!a_neumfac.box().isEmpty());
  assert(!a_dircfac.box().isEmpty());
  assert(!a_inhmval.box().isEmpty());
  a_neumfac.setVal(0.0);
  a_dircfac.setVal(1.0);
  a_inhmval.setVal(m_scalType);
}




BoxGhostBC*
scalarInflowBC::new_boxghostbc() const
{

  scalarInflowBC* newop = new scalarInflowBC();
  if(newop == NULL)
    {
      MayDay::Error("Out of Memory in scalarInflowBC::new_boxghostbc");
    }

  newop->setScalarType(m_scalType);
  return static_cast<BoxGhostBC*>(newop);
}
