/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// AMRNavierStokes.cpp

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000


/*****************/
/*****************/

#ifndef _AMRNavierStokeHeaders_H_
#define _AMRNavierStokeHeaders_H_

#include <iostream>
using std::cout;
using std::cin;
using std::cerr;
#include <cmath>
#include <cstdio>

#include "parstream.H"
#include "LoadBalance.H"
#include "LayoutIterator.H"
#include "FineInterp.H"
#include "ParmParse.H"
#include "LoHiSide.H"
#include "BoxIterator.H"
#include "ProblemDomain.H"
#include "AMRIO.H"

#ifdef HDF5
#include "HDF5.H"
#endif

#include "AMRNavierStokes.H"
#include "GodunovTrace.H"
#include "basicVelBC.H"
#include "ExtrapBC.H"
#include "HOExtrapBC.H"
#include "PoissonBC.H"
#include "scalarDirichletBC.H"
#include "viscousBC.H"
#include "Divergence.H"
#include "CellToEdge.H"
#include "EdgeToCell.H"
#include "computeNorm.H"
#include "IAMRGodunovF_F.H"
#include "PROB_F_F.H"
#include "AMRNS_F_F.H"
#include "timeInterp.H"
#include "Gradient.H"

#include "Timer.H"

#ifdef DEBUG
#include "computeSum.H"
#endif

#ifdef MATCH_OLDCODE
// this is necessary because we call library fortran directly
#include "PoissonOpF_F.H"
#endif

// #define TEST_TAGGING  

#define ADVECT_GROW 2

#ifdef CH_USE_DOUBLE
#define TIME_EPS 1.0e-10
#else
#define TIME_EPS 1.0e-5
#endif

// small parameter used in defining TGA constants for viscous solves
#define TGA_EPS 1.0e-8

//#include "AMRNavierStokesTimers.H"

extern Timer TimeNSadvance;           
extern Timer TimeNSadvanceIntro            ;
extern Timer TimeNSadvanceCompAV           ;
extern Timer TimeNSadvanceAdvectLambda     ;
extern Timer TimeNSadvanceAdvectDiffScalar ;
extern Timer TimeNSadvancePredictVelocities;
extern Timer TimeNSadvanceCompUStar        ;
extern Timer TimeNSadvanceLevelProject     ;

extern Timer cs0;
extern Timer cs1;
extern Timer cs2;
extern Timer cs3;
extern Timer cs4;
extern Timer cs5;
extern Timer cs7;
extern Timer cs8;
extern Timer TimeNScompUStarlevelSolve1;
extern Timer cs10;
extern Timer cs11;
extern Timer TimeNScompUStarlevelSolve2;
extern Timer cs13;
extern Timer cs14;
extern Timer cs15;

extern Timer TimeNSpost;
extern Timer TimeNSpostPart1;
extern Timer TimeNSpostPart2;
extern Timer TimeNSpostImplicitReflux;
extern Timer TimeNSpostSyncOperations;
extern Timer TimeNSpostDiagnostic;
extern Timer TimeNSpostCreateBC;

extern Timer TimeNSpostRegrid;
extern Timer TimeinitialVelocityProject;
extern Timer TimedoPostRegridOps;
extern Timer TimeinitializeGlobalPressure;

extern Timer TimecomputeAdvectionVelocities;
//extern Timer TimeSetPBC;
extern Timer TimeTraceState;
extern Timer TimelevelMacProject;

extern Timer NST_Everything;
extern Timer TimeSA_diffusionSolver;
extern Timer TimeSA_viscousSolver;
extern Timer TimeSA_initialData;
extern Timer TimeSA_velocity;
extern Timer TimeSA_scalar;

extern Timer TimeLS_advectScalar1;
extern Timer TimeLS_advectScalar2;
extern Timer TimeLS_compUstar1;
extern Timer TimeLS_compUstar2;
extern Timer TimeLS_compUstarInit1; //noel
extern Timer TimeLS_compUstarInit2; //noel


// AMRNavierStokeInit.cpp
extern Timer TF_INITVORT;
extern Timer TF_COMPUTEVORT;
extern Timer TF_INITSCAL;

// AMRNavierStokeVel.cpp
extern Timer TF_OPERATORLAP_ustar;
extern Timer TF_OPERATORLAP_ustarInit;
extern Timer TF_UDELS_predictVelocities;

// AMRNavierStokeScal.cpp
extern Timer TF_OPERATORLAP_advectDiffuseScalar;


#endif
