/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// LevelHelmholtzSolver.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _LevelHelmholtzSolver_H_
#define _LevelHelmholtzSolver_H_

#include <LevelSolver.H>
#include <HelmholtzOp.H>

/// special case LevelSolver to solve Helmholtz eqn 
/** LevelHelmholtzSolver is a special case of LevelSolver
    for solving the Helmholtz equation on a level.  It
    allows the coefficients to be changed without allocating
    an entire new LevelSolver. */
class LevelHelmholtzSolver: public LevelSolver
{

public:
  /// Default constructor
  LevelHelmholtzSolver();

  virtual ~LevelHelmholtzSolver();

  /// constructor -- note that there is no need to pass in a LevelOp,
  LevelHelmholtzSolver(const DisjointBoxLayout& a_grids,
		       const DisjointBoxLayout* a_baseGrids,
		       const ProblemDomain& a_domain,
		       Real a_dxLevel, 
		       int a_nRefCrse);

  /// (deprecated) constructor -- note there is no need to pass in a LevelOp,
  LevelHelmholtzSolver(const DisjointBoxLayout& a_grids,
		       const DisjointBoxLayout* a_baseGrids,
		       const Box& a_domain,
		       Real a_dxLevel, 
		       int a_nRefCrse);

  ///
  LevelHelmholtzSolver(const DisjointBoxLayout& a_grids,
		       const DisjointBoxLayout* a_baseGrids,
		       const ProblemDomain& a_domain,
		       Real a_dxLevel, 
		       int a_nRefCrse,
		       const HelmholtzOp* const a_opin);

  /// deprecated interface using Box instead of ProblemDomain
  LevelHelmholtzSolver(const DisjointBoxLayout& a_grids,
		       const DisjointBoxLayout* a_baseGrids,
		       const Box& a_domain,
		       Real a_dxLevel, 
		       int a_nRefCrse,
		       const HelmholtzOp* const a_opin);

  ///
  void define(const DisjointBoxLayout& a_grids,
	      const DisjointBoxLayout* a_baseGrids,
	      const ProblemDomain& a_domain,
	      Real a_dxLevel, 
	      int a_nRefCrse);

  /// deprecated interface using Box instead of ProblemDomain
  void define(const DisjointBoxLayout& a_grids,
	      const DisjointBoxLayout* a_baseGrids,
	      const Box& a_domain,
	      Real a_dxLevel, 
	      int a_nRefCrse);

  ///
  virtual void define(const DisjointBoxLayout& a_grids, 
		      const DisjointBoxLayout* a_baseGrids,
		      const ProblemDomain& a_domain, 
		      Real a_dxLevel,
		      int a_nRefCrse,
		      const HelmholtzOp* const a_opin);


  /// (deprecated) interface using Box instead of ProblemDomain
  virtual void define(const DisjointBoxLayout& a_grids, 
		      const DisjointBoxLayout* a_baseGrids,
		      const Box& a_domain, 
		      Real a_dxLevel,
		      int a_nRefCrse,
		      const HelmholtzOp* const a_opin);


  /// this function hides the LevelSolver version -- should never be called
  virtual void define(const DisjointBoxLayout& a_grids, 
		      const DisjointBoxLayout* a_baseGrids,
		      const ProblemDomain& a_domain, 
		      Real a_dxLevel,
		      int a_nRefCrse,
		      const LevelOp* const a_opin);

  /// this function hides the LevelSolver version -- should never be called
  virtual void define(const DisjointBoxLayout& a_grids, 
		      const DisjointBoxLayout* a_baseGrids,
		      const Box& a_domain, 
		      Real a_dxLevel,
		      int a_nRefCrse,
		      const LevelOp* const a_opin);


  ///
  void setBeta(Real a_beta);

  ///
  void setBC(const DomainGhostBC& a_BC);



};


#endif


