/* _______              __
  / ___/ /  ___  __ _  / /  ___
 / /__/ _ \/ _ \/  ' \/ _ \/ _ \
 \___/_//_/\___/_/_/_/_.__/\___/ 
*/

// channelBC.H

//
// This software is copyright (C) by the Lawrence Berkeley
// National Laboratory.  Permission is granted to reproduce
// this software for non-commercial purposes provided that
// this notice is left intact.
// 
// It is acknowledged that the U.S. Government has rights to
// this software under Contract DE-AC03-765F00098 between
// the U.S.  Department of Energy and the University of
// California.
//
// This software is provided as a professional and academic
// contribution for joint exchange. Thus it is experimental,
// is provided ``as is'', with no warranties of any kind
// whatsoever, no support, no promise of updates, or printed
// documentation. By using this software, you acknowledge
// that the Lawrence Berkeley National Laboratory and
// Regents of the University of California shall have no
// liability with respect to the infringement of other
// copyrights by any part of this software.
//
// Dan Martin, Fri, Jan 14, 2000

#ifndef _channelBC_H_
#define _channelBC_H_

#include "PhysBCUtil.H"
#include "scalarDirichletBC.H"
#include "basicVelBC.H"
#include "ExtrapBC.H"
#include "PoissonBC.H"


/// specific BC's for the convection problem (particularly for scalars)
class channelBC : public PhysBCUtil
{

public:
  /// constructor
  channelBC();

  /// destructor
  virtual ~channelBC();

  /// "virtual" constructor
  virtual PhysBCUtil* newPhysBCUtil() const;

  /// only BCs which needs to be overwritten are scalar BCs
  

  ///
  void setMaxInflowVel(Real a_maxInflowVel);

  /// 
  Real maxInflowVel() const;

  /// interact with ParmParse to set physical BC types
  virtual void setBCs();
  

  /// compute effect of wall heating on dt
  virtual void computeBoundaryDt(Real& a_dt, Real a_cfl, Real a_dx) const;


  ///
  virtual DomainGhostBC* gradMacPressureBC() const;

  ///
  virtual DomainGhostBC* gradELambdaBC() const;

  /// physical bc's for diffusion term for scalars
  virtual DomainGhostBC* diffusiveBC(int a_scalarType) const;


protected:

  ///
  virtual DomainGhostBC* basicScalBC(bool a_isHomogeneous,
				     int a_scalType) const;
  

  ///
  virtual void basicCCVelBC(DomainGhostBC& a_newBC,
			    bool a_isHomogeneous,
			    bool a_isViscous,
			    int a_velComponent,
			    Interval& a_interval) const;

  ///
  virtual basicEdgeVelBC* basicECVelBC(bool a_isHomogeneous,
				       bool a_isViscous,
				       bool a_isSingleComponent) const;

  /// max inflow velocity
  Real m_maxInflowVel;

private:

  /// copy constructor
  channelBC(const channelBC& rhs) {;}

};


///
class scalarInflowChannelBC : public scalarInflowBC
{
  
public:
  friend class DomainGhostBC;

  ///
  scalarInflowChannelBC();

  ///
  scalarInflowChannelBC(int dir, Side::LoHiSide sd);

  ///
  scalarInflowChannelBC(int dir, Side::LoHiSide sd,
			const Interval& a_comps);

  ///
  virtual ~scalarInflowChannelBC();

private:
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dirfac,
			    FArrayBox& a_inhmval,
			    Real a_dx,
			    const Box& a_domain) const;

  virtual BoxGhostBC* new_boxghostbc() const;


};


///
class NormalInflowVelBC : public InflowVelBC
{

public:
  friend class DomainGhostBC;

  ///
  NormalInflowVelBC();

  ///
  virtual ~NormalInflowVelBC();

  ///
  NormalInflowVelBC(int dir, Side::LoHiSide sd);

  ///
  NormalInflowVelBC(int dir, Side::LoHiSide sd,
		    const Interval& a_comps);


protected:
  
  /// 
  virtual void fillBCValues(FArrayBox& a_neumfac,
			    FArrayBox& a_dirfac,
			    FArrayBox& a_inhmval,
			    Real a_dx,
			    const Box& a_domain) const;

  ///
  virtual BoxGhostBC* new_boxghostbc() const;

private:

  NormalInflowVelBC(const NormalInflowVelBC& bcin) {;}
  void operator =(const NormalInflowVelBC& bcin) {;}

};




#endif


